/**
 * \file
 *
 * \brief ADC Basic driver implementation.
 *
 (c) 2020 Microchip Technology Inc. and its subsidiaries.

    Subject to your compliance with these terms,you may use this software and
    any derivatives exclusively with Microchip products.It is your responsibility
    to comply with third party license terms applicable to your use of third party
    software (including open source software) that may accompany Microchip software.

    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
    WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
    PARTICULAR PURPOSE.

    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
    BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
    FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
    ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
    THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 */

/**
 * \defgroup doc_driver_adc_basic ADC Basic Driver
 * \ingroup doc_driver_adc
 *
 * \section doc_driver_adc_rev Revision History
 * - v0.0.0.1 Initial Commit
 *
 *@{
 */
#include <adc_pga.h>

/** Function pointer to callback function called by IRQ.
    NULL=default value: No callback function is to be used.
*/

adc_irq_cb_t ADC_0_cb = NULL; /**
                               * \brief Initialize ADC interface
                               * If module is configured to disabled state, the clock to the ADC is disabled
                               * if this is supported by the device's clock system.
                               *
                               * \return Initialization status.
                               * \retval 0 the ADC init was successful
                               * \retval 1 the ADC init was not successful
                               */
int8_t ADC_0_init()
{

	// ADC0.CTRLB = ADC_PRESC_DIV2_gc; /* System clock divided by 2 */

	// ADC0.CTRLF = ADC_SAMPNUM_NONE_gc /* No accumulation */
	//		 | 0 << ADC_FREERUN_bp; /* ADC Freerun mode: disabled */

	ADC0.CTRLC = ADC_REFSEL_2500MV_gc     /* VDD */
	| ADC_TIMEBASE_VALUE; /* timebase value */

	ADC0.CTRLE = 0xf; /* Sample Duration: 0x1 */

	// ADC0.DBGCTRL = 0 << ADC_DBGRUN_bp; /* Debug run: disabled */

	ADC0.COMMAND = 1 << ADC_DIFF_bp           /* Differential ADC Conversion: enabled */
	| ADC_MODE_SINGLE_12BIT_gc /* Single Conversion 12-bit */
	| ADC_START_STOP_gc;       /* Stop an ongoing conversion */

	ADC0.INTCTRL = 1 << ADC_RESRDY_bp /* Result Ready Interrupt Enable: enabled */;

	ADC0.MUXPOS = ADC_VIA_PGA_gc        /* Via PGA */
	| ADC_MUXPOS_AIN2_gc; /* ADC input pin 2 */

	ADC0.MUXNEG = ADC_VIA_PGA_gc        /* Via PGA */
	| ADC_MUXNEG_AIN1_gc; /* ADC input pin 1 */

	ADC0.CTRLA = 1 << ADC_ENABLE_bp      /* ADC Enable: enabled */
	| 0 << ADC_RUNSTDBY_bp; /* Run standby mode: disabled */

	ADC0.PGACTRL = 1 << ADC_PGAEN_bp             /* PGA Enable: enabled */
	| ADC_GAIN_1X_gc              /* 1X Gain */
	| ADC_PGABIASSEL_1X_gc        /* 1x BIAS current */
	| ADC_ADCPGASAMPDUR_20CLK_gc; /* 20 ADC cycles */

	return 0;
}

/**
 * \brief Enable ADC_0
 * 1. If supported by the clock system, enables the clock to the ADC
 * 2. Enables the ADC module by setting the enable-bit in the ADC control register
 *
 * \return Nothing
 */
void ADC_0_enable()
{
	ADC0.CTRLA |= ADC_ENABLE_bm;
}
/**
 * \brief Disable ADC_0
 * 1. Disables the ADC module by clearing the enable-bit in the ADC control register
 * 2. If supported by the clock system, disables the clock to the ADC
 *
 * \return Nothing
 */
void ADC_0_disable()
{
	ADC0.CTRLA &= ~ADC_ENABLE_bm;
}

/**
 * \brief Start a conversion on ADC_0
 *
 * \param[in] channel The ADC channel to start conversion on
 *
 * \return Nothing
 */
void ADC_0_start_conversion(adc_0_channel_t channel)
{
	ADC0.MUXPOS = channel;
	ADC0.COMMAND |= ADC_START_IMMEDIATE_gc;
}

/**
 * \brief Check if the ADC conversion is done
 *
 * \return The status of ADC converison done check
 * \retval true The ADC conversion is done
 * \retval false The ADC converison is not done
 */
bool ADC_0_is_conversion_done()
{
	return (ADC0.INTFLAGS & ADC_RESRDY_bm);
}

/**
 * \brief Read a conversion result from ADC_0
 *
 * \return Conversion result read from the ADC_0 ADC module
 */
adc_result_t ADC_0_get_conversion_result(void)
{
	return (ADC0.RESULT);
}

/**
 * \brief Start a conversion, wait until ready, and return the conversion result
 *
 * \return Conversion result read from the ADC_0 ADC module
 */
adc_result_t ADC_0_get_conversion(adc_0_channel_t channel)
{
	adc_result_t res;

	ADC_0_start_conversion(channel);
	while (!ADC_0_is_conversion_done())
		;
	res = ADC_0_get_conversion_result();
	ADC0.INTFLAGS |= ADC_RESRDY_bm;
	return res;
}

/**
 * \brief Return the number of bits in the ADC conversion result
 *
 * \return The number of bits in the ADC conversion result
 */
uint8_t ADC_0_get_resolution()
{
	return (ADC0.COMMAND & ADC_MODE_SINGLE_8BIT_gc) ? 8 : 12;
}

/**
 * \brief Register a callback function to be called at the end of the ADC ISR.
 *
 * \param[in] f Pointer to function to be called
 *
 * \return Nothing.
 */
void ADC_0_register_callback(adc_irq_cb_t f)
{
	ADC_0_cb = f;
}

 int32_t  Pactive_i;
 extern int32_t P_active_t, Irms_t, Vrms_t;
 //extern int64_t Vrms_avg, Irms_avg, P_active_avg;

 volatile int16_t temp;
 volatile float angle =0;
 volatile uint8_t curr_volt_channel_flag =0, cycle_finished =0;
 const float sine_speed = M_PI_4/10;
 extern int16_t sample_counter;
 #define ZERO_CROSS_THERSHOLD 5
 #define ZERO_CROSS_HYST 40
 uint8_t hysteresis_counter =0;
 extern int16_t i_inst, v_inst, old_v_inst;
bool zero_crossing_flag = false;
 ISR(ADC0_RESRDY_vect)
 {
	 // Clear the interrupt flag

	 if (ADC_0_cb != NULL) {
		 ADC_0_cb();
	 }
	 if(curr_volt_channel_flag==0)
	 {
		 sample_counter++;
		 //if(counter == 250 )
		 //{
		 //cycle_finished =1;
		 //
		 //counter=0;
		 //}
		 //temp ++;//= sin(angle) * 32767;
		//if ((int16_t)ADC0.RESULT > v_inst)
			v_inst  = (int16_t)ADC0.RESULT;



		
		
		/***************** NO PGA *****************/
		// Voltage Channel PA4 & PA5
		ADC0.MUXPOS = ADC_VIA_ADC_gc        /* Via PGA */
		| ADC_MUXPOS_AIN2_gc; /* ADC input pin 5 */

		ADC0.MUXNEG = ADC_VIA_ADC_gc        /* Via PGA */
		| ADC_MUXNEG_AIN1_gc; /* ADC input pin 4 */
		ADC0.CTRLE = 0x0f; /* Sample Duration: 0x1 */		 
		//Gain = X
		//ADC0.PGACTRL = 0 << ADC_PGAEN_bp             /* PGA Enable: enabled */
		//| ADC_GAIN_2X_gc              /* 2X Gain */
		//| ADC_PGABIASSEL_1X_gc        /* 1x BIAS current */
		//| ADC_ADCPGASAMPDUR_32CLK_gc; /* 20 ADC cycles */

		/***************** with PGA *****************/
		 // Current Channel PA1 & PA2
		 //ADC0.MUXPOS = ADC_VIA_PGA_gc       /* Via PGA */
		 //| ADC_MUXPOS_AIN2_gc; /* ADC input pin 2 */
		 //
		 //ADC0.MUXNEG = ADC_VIA_PGA_gc        /* Via PGA */
		 //| ADC_MUXNEG_AIN1_gc; /* ADC input pin 1 */
		 //
		 ////Gain = 1X
		 //ADC0.PGACTRL = 1 << ADC_PGAEN_bp             /* PGA Enable: enabled */
		 //| ADC_GAIN_1X_gc              /* 8X Gain */
		 //| ADC_PGABIASSEL_1X_gc        /* 1x BIAS current */
		 //| ADC_ADCPGASAMPDUR_20CLK_gc; /* 20 ADC cycles */
			//
		
		
		
		
		 curr_volt_channel_flag = 1;// ADC channel set to current inputs
		 ADC0.COMMAND |= ADC_START_IMMEDIATE_gc;
		 // ADC_0_get_conversion(); // Convert current value
		 

	 }
	 else
	 {
		 i_inst = (int16_t)ADC0.RESULT;

		 Pactive_i = (int32_t)i_inst *  (int32_t)v_inst;
		 P_active_t += Pactive_i;
		 Irms_t += pow(i_inst,2);
		 Vrms_t += pow(v_inst,2);

		 if(v_inst <= 0 && old_v_inst > 0)
		 {
			 zero_crossing_flag = true; // Zero-crossing detected
		 }
		 if(!zero_crossing_flag)
		 old_v_inst = v_inst;
		 else {
			 hysteresis_counter++;
			 if(hysteresis_counter> ZERO_CROSS_HYST)
			 {
				 if(v_inst <= 0 && old_v_inst > 0 ){
					 //VPORTC.OUT ^= (1 << 1);
					 cycle_finished =1;
					 zero_crossing_flag = false;
					 hysteresis_counter =0;
					 old_v_inst = v_inst;
					 return;
				 }
			 }
		 }

		 /***************** NO PGA *****************/
		 // Voltage Channel PA4 & PA5
		 //ADC0.MUXPOS = ADC_VIA_ADC_gc        /* Via PGA */
		 //| ADC_MUXPOS_AIN5_gc; /* ADC input pin 5 */
//
		 //ADC0.MUXNEG = ADC_VIA_ADC_gc        /* Via PGA */
		 //| ADC_MUXNEG_AIN4_gc; /* ADC input pin 4 */
		//ADC0.CTRLE = 0x1b; /* Sample Duration: 0x1b */
 
		 //Gain = X
		 //ADC0.PGACTRL = 0 << ADC_PGAEN_bp             /* PGA Enable: enabled */
		 //| ADC_GAIN_2X_gc              /* 2X Gain */
		 //| ADC_PGABIASSEL_1X_gc        /* 1x BIAS current */
		 //| ADC_ADCPGASAMPDUR_32CLK_gc; /* 20 ADC cycles */


		/***************** with PGA *****************/
		 // Voltage Channel PA4 & PA5
		ADC0.MUXPOS = ADC_VIA_PGA_gc        /* Via PGA */
		| ADC_MUXPOS_AIN5_gc; /* ADC input pin 5 */

		ADC0.MUXNEG = ADC_VIA_PGA_gc        /* Via PGA */
		| ADC_MUXNEG_AIN4_gc; /* ADC input pin 4 */
		 ADC0.CTRLE = 0x1b; /* Sample Duration: 0x1b */
		 ////Gain = X
		ADC0.PGACTRL = 1 << ADC_PGAEN_bp             /* PGA Enable: enabled */
		| ADC_GAIN_4X_gc              /* 2X Gain */
		| ADC_PGABIASSEL_1X_gc        /* 1x BIAS current */
		| ADC_ADCPGASAMPDUR_32CLK_gc; /* 20 ADC cycles */
		
		
		
		
		curr_volt_channel_flag = 0;// ADC channel set to voltage inputs
		 ADC0.COMMAND |= ADC_START_IMMEDIATE_gc;

	 }

	 //VPORTB.OUT &= ~(1 <<6);
	 ADC0.INTFLAGS |= ADC_RESRDY_bm;
 }
